const path = require('path');
const moduleToCdn = require('@talend/module-to-cdn');
const DynamicCdnWebpackPlugin = require('@talend/dynamic-cdn-webpack-plugin');

function init()  {
    moduleToCdn.add({
        '@talend/react-components': {
            var: 'TalendReactComponents',
            versions: {
                '>= 6.0.0-alpha.10': {
                    development: '/dist/TalendReactComponents.js',
                    production: '/dist/TalendReactComponents.min.js',
                }
            }
        },
        '@talend/react-forms': {
            var: 'TalendReactForms',
            versions: {
                '>= 6.0.0-alpha.10': {
                    development: '/dist/TalendReactForms.js',
                    production: '/dist/TalendReactForms.min.js',
                }
            }
        },
        '@talend/react-cmf': {
            var: 'TalendReactCmf',
            versions: {
                '>= 6.0.0-alpha.10': {
                    development: '/dist/TalendReactCmf.js',
                    production: '/dist/TalendReactCmf.min.js',
                }
            }
        },
        '@talend/react-cmf-router': {
            var: 'TalendReactCmfRouter',
            versions: {
                '>= 3.4.0-alpha-1': {
                    development: '/dist/TalendReactCmfRouter.js',
                    production: '/dist/TalendReactCmfRouter.min.js',
                }
            }
        },
        '@talend/react-containers': {
            var: 'TalendReactContainers',
            versions: {
                '>= 6.0.0-alpha.10': {
                    development: '/dist/TalendReactContainers.js',
                    production: '/dist/TalendReactContainers.min.js',
                }
            }
        },
        '@talend/react-datagrid': {
            var: 'TalendReactDatagrid',
            versions: {
                '>= 6.0.0-alpha.10': {
                    development: '/dist/TalendReactDatagrid.js',
                    production: '/dist/TalendReactDatagrid.min.js',
                }
            }
        },
        '@talend/icons/dist/react': {
            var: 'TalendReactIcons',
            versions: {
                '>= 6.0.0-alpha.10': {
                    development: '/dist/TalendReactIcons.js',
                    production: '/dist/TalendReactIcons.min.js',
                }
            }
        },
        '@talend/icons/dist/info': {
            var: 'TalendInfoIcons',
            versions: {
                '>= 6.0.0-alpha.10': {
                    development: '/dist/TalendInfoIcons.js',
                    production: '/dist/TalendInfoIcons.min.js',
                }
            }
        },
        // to let peerdeps in webpack work
        '@talend/icons': {
            var: 'TalendInfoIcons',
            versions: {
                '>= 6.0.0-alpha.10': {
                    development: '/dist/TalendInfoIcons.js',
                    production: '/dist/TalendInfoIcons.min.js',
                }
            }
        },
    });
}

function getCopyConfig() {
    const config = [];
    // TODO: support yarn.lock file and move all that code in the webpack dynmaic cdn;
    const packagelock = require(path.join(process.cwd(), 'package-lock.json'));
    Object.keys(packagelock.dependencies).forEach(moduleName => {
        const info = moduleToCdn(moduleName, packagelock.dependencies[moduleName].version, { env: 'development' });
        if (info && info.local) {
            config.push({ from: info.local, to: `cdn/${info.name}/${info.version}${info.path}`});
        }
    });
	return config;
}

function resolver(...args) {
    // will change once we have publish our CDN
	return moduleToCdn(...args);
}

function localResolver(...args) {
    const info = moduleToCdn(...args);
    if (!info) {
        return info;
    }
    info.url = `/cdn/${info.name}/${info.version}${info.path}`;
	return info;
}

function getWebpackPlugin(env) {
    let loglevel = env.cdnlog;
    // option is passed throw cmd line and env by ui-scripts-core.
    if (Array.isArray(env.cdnlog)) {
        loglevel = env.cdnlog[0];
    }
    return new DynamicCdnWebpackPlugin({ resolver: env.cdn ? resolver : localResolver, loglevel });
}

module.exports = {
    resolver,
    localResolver,
    init,
    getCopyConfig,
    getWebpackPlugin,
};